@@ -127,6 +127,11 @@
                "https://magit.vc/manual/forge/How-Forge-Detection-Works.html"))
     remote))
 
+(defun forge--get-default-branch (repo)
+  (let ((default-directory (forge-get-worktree repo)))
+    (concat (forge--get-remote) "/"
+            (or (oref repo default-branch) "master"))))
+
 (cl-defmethod forge-get-repository ((_(eql :id)) id)
   (closql-get (forge-db) (substring-no-properties id) 'forge-repository))
 
@@ -146,46 +151,46 @@ then return that repository.
 Otherwise return the repository for `default-directory', if that
 exists and satisfies DEMAND.  If that fails too, then return nil
 or signal an error, depending on DEMAND."
-  (or (and-let* ((repo (and (not notatpt)
-                            (forge-repository-at-point))))
+  (or (and-let ((_(not notatpt))
+                (repo (forge-repository-at-point)))
         (forge-get-repository repo 'noerror demand))
-      (and-let* ((repo (or (forge-buffer-repository)
-                           (and forge-buffer-topic
-                                (forge-get-repository forge-buffer-topic)))))
+      (and-let ((_(not remote))
+                (repo (or (forge-buffer-repository)
+                          (and forge-buffer-topic
+                               (forge-get-repository forge-buffer-topic)))))
         (forge-get-repository repo 'noerror demand))
       (magit--with-refresh-cache
           (list default-directory 'forge-get-repository demand)
-        (if (not (magit-gitdir))
-            (when (memq demand forge--signal-no-entry)
-              (error
-               "Cannot determine Forge repository outside of Git repository"))
-          (unless remote
-            (setq remote (forge--get-remote 'warn)))
-          (if-let ((url (and remote
-                             (magit-git-string "remote" "get-url" remote))))
-              (and-let* ((repo (forge-get-repository url remote demand)))
-                (progn ; work around debbugs#31840
-                  (oset repo worktree (magit-toplevel))
-                  repo))
-            (when (memq demand forge--signal-no-entry)
-              (error
-               "Cannot determine forge repository.  %s\nSee %s."
-               (cond (remote (format "No url configured for %S." remote))
-                     ((and-let* ((config (magit-get "forge.remote")))
-                        (format "Value of `forge.remote' is %S but %s"
-                                config "that remote does not exist.")))
-                     ((magit-list-remotes) "Cannot decide on remote to use.")
-                     (t "No remote configured."))
-               "https://magit.vc/manual/forge/How-Forge-Detection-Works.html"
-               )))))))
+        (cond-let
+          ((not (magit-gitdir))
+           (when (memq demand forge--signal-no-entry)
+             (error
+              "Cannot determine Forge repository outside of Git repository")))
+          [[remote (or remote (forge--get-remote 'warn))]]
+          ([_ remote]
+           [url (magit-git-string "remote" "get-url" remote)]
+           (and$ (forge-get-repository url remote demand)
+                 (prog1 $ (oset $ worktree (magit-toplevel)))))
+          ((memq demand forge--signal-no-entry)
+           (error
+            "Cannot determine forge repository.  %s\nSee %s."
+            (cond (remote (format "No url configured for %S." remote))
+                  ((and$ (magit-get "forge.remote")
+                         (format "Value of `forge.remote' is %S but %s"
+                                 $ "that remote does not exist.")))
+                  ((magit-list-remotes) "Cannot decide on remote to use.")
+                  (t "No remote configured."))
+            "https://magit.vc/manual/forge/How-Forge-Detection-Works.html"
+            ))))))
 
 (cl-defmethod forge-get-repository ((url string) &optional remote demand)
   "Return the repository at URL."
-  (if-let ((parts (forge--split-forge-url url)))
-      (forge-get-repository parts remote (or demand :known?))
-    (when (memq demand forge--signal-no-entry)
-      (error "Cannot determine forge repository.  %s isn't a forge URL.  %s"
-             url "You might have to customize `forge-alist'."))))
+  (cond-let
+    ([parts (forge--split-forge-url url)]
+     (forge-get-repository parts remote (or demand :known?)))
+    ((memq demand forge--signal-no-entry)
+     (error "Cannot determine forge repository.  %s isn't a forge URL.  %s"
+            url "You might have to customize `forge-alist'."))))
 
 (cl-defmethod forge-get-repository (((host owner name) list)
                                     &optional remote demand)
@@ -196,75 +201,75 @@ See `forge-alist' for valid Git hosts."
   (setq host  (substring-no-properties host))
   (setq owner (substring-no-properties owner))
   (setq name  (substring-no-properties name))
-  (unless (memq demand '( :tracked :tracked?
-                          :known? :insert! :valid?
-                          :stub :stub?))
-    (if-let ((new (pcase demand
-                    ('t      :tracked)
-                    ('full   :tracked?)
-                    ('nil    :known?)
-                    ('create :insert!)
-                    ('stub   :stub)
-                    ('maybe  :stub?))))
-        (progn
-          (message "Obsolete value for `%s's DEMAND: `%s'; use `%s' instead"
-                   'forge-get-repository demand new)
-          (setq demand new))
-      (error "Unknown value for `%s's DEMAND: `%s'"
-             'forge-get-repository demand)))
-  (if-let ((spec (forge--get-forge-host host t)))
-      (pcase-let ((`(,githost ,apihost ,webhost ,class) spec))
-        ;; The `webhost' is used to identify the corresponding forge.
-        ;; For that reason it is stored in the `forge' slot.  The id
-        ;; stored in the `id' slot also derives from that value.
-        (let* ((row (car (forge-sql [:select * :from repository
-                                     :where (and (= forge $s1)
-                                                 (= owner $s2)
-                                                 (= name  $s3))]
-                                    webhost owner name)))
-               (obj (and row (closql--remake-instance class (forge-db) row))))
-          ;; Synchronize the object with the entry from `forge-alist'.
-          ;; This only has an effect if the entry was modified, which
-          ;; should rarely, if ever, happen.  Avoid confusion, by not
-          ;; mentioning this detail in any docstring.
-          (when obj
-            (oset obj apihost apihost)
-            (oset obj githost githost)
-            (oset obj remote  remote))
-          (pcase (list demand (and obj (eq (oref obj condition) :tracked)))
-            (`(:tracked? nil) (setq obj nil))
-            (`(:tracked  nil)
-             (error "Cannot use `%s' in %S yet.\n%s"
-                    this-command (magit-toplevel)
-                    "Use `M-x forge-add-repository' before trying again.")))
-          (when (and (memq demand '(:insert! :valid? :stub :stub?))
-                     (not obj))
-            (pcase-let ((`(,id . ,forge-id)
-                         (forge--repository-ids
-                          class webhost owner name
-                          (memq demand '(:stub :stub?))
-                          (eq demand :valid?))))
-              (if (not id)
-                  ;; `:valid?' was used and it turned out it is not.
-                  (setq obj nil)
-                ;; The repo might have been renamed on the forge.  #188
-                (unless (setq obj (forge-get-repository :id id))
-                  (setq obj (funcall class
-                                     :id       id
-                                     :forge-id forge-id
-                                     :forge    webhost
-                                     :owner    owner
-                                     :name     name
-                                     :apihost  apihost
-                                     :githost  githost
-                                     :remote   remote))
-                  (when (eq demand :insert!)
-                    (closql-insert (forge-db) obj)
-                    (oset obj condition :known))))))
-          obj))
-    (when (memq demand forge--signal-no-entry)
-      (error "Cannot determine forge repository.  No entry for %S in %s"
-             host 'forge-alist))))
+  (cond-let
+    ((memq demand '(:tracked :tracked? :known? :insert! :valid? :stub :stub?)))
+    ([corrected (pcase demand
+                  ('t      :tracked)
+                  ('full   :tracked?)
+                  ('nil    :known?)
+                  ('create :insert!)
+                  ('stub   :stub)
+                  ('maybe  :stub?))]
+     (message "Obsolete value for `%s's DEMAND: `%s'; use `%s' instead"
+              'forge-get-repository demand corrected)
+     (setq demand corrected))
+    ((error "Unknown value for `%s's DEMAND: `%s'"
+            'forge-get-repository demand)))
+  (cond-let
+    ([spec (forge--get-forge-host host t)]
+     (pcase-let*
+         ;; The `webhost' is used to identify the corresponding forge.
+         ;; For that reason it is stored in the `forge' slot.  The id
+         ;; stored in the `id' slot also derives from that value.
+         ((`(,githost ,apihost ,webhost ,class) spec)
+          (row (car (forge-sql [:select * :from repository
+                                :where (and (= forge $s1)
+                                            (= owner $s2)
+                                            (= name  $s3))]
+                               webhost owner name)))
+          (obj (and row (closql--remake-instance class (forge-db) row))))
+       ;; Synchronize the object with the entry from `forge-alist'.
+       ;; This only has an effect if the entry was modified, which
+       ;; should rarely, if ever, happen.  Avoid confusion, by not
+       ;; mentioning this detail in any docstring.
+       (when obj
+         (oset obj apihost apihost)
+         (oset obj githost githost)
+         (oset obj remote  remote))
+       (pcase (list demand (and obj (eq (oref obj condition) :tracked)))
+         (`(:tracked? nil) (setq obj nil))
+         (`(:tracked  nil)
+          (error "Cannot use `%s' in %S yet.\n%s"
+                 this-command (magit-toplevel)
+                 "Use `M-x forge-add-repository' before trying again.")))
+       (when (and (memq demand '(:insert! :valid? :stub :stub?))
+                  (not obj))
+         (pcase-let ((`(,id . ,forge-id)
+                      (forge--repository-ids
+                       class webhost owner name
+                       (memq demand '(:stub :stub?))
+                       (eq demand :valid?))))
+           (if (not id)
+               ;; `:valid?' was used and it turned out it is not.
+               (setq obj nil)
+             ;; The repo might have been renamed on the forge.  #188
+             (unless (setq obj (forge-get-repository :id id))
+               (setq obj (funcall class
+                                  :id       id
+                                  :forge-id forge-id
+                                  :forge    webhost
+                                  :owner    owner
+                                  :name     name
+                                  :apihost  apihost
+                                  :githost  githost
+                                  :remote   remote))
+               (when (eq demand :insert!)
+                 (closql-insert (forge-db) obj)
+                 (oset obj condition :known))))))
+       obj))
+    ((memq demand forge--signal-no-entry)
+     (error "Cannot determine forge repository.  No entry for %S in %s"
+            host 'forge-alist))))
 
 (cl-defmethod forge-get-repository ((repo forge-repository)
                                     &optional noerror demand)
@@ -304,33 +309,33 @@ See `forge-alist' for valid Git hosts."
 If there is no such repository and DEMAND is non-nil, then signal
 an error."
   (or (magit-section-value-if 'forge-repo)
-      (and-let* ((topic (forge-topic-at-point)))
+      (and-let ((topic (forge-topic-at-point)))
         (forge-get-repository topic))
-      (and (derived-mode-p 'forge-repository-list-mode)
-           (and-let* ((id (tabulated-list-get-id)))
-             (forge-get-repository :id id)))
-      (and (derived-mode-p 'magit-repolist-mode)
-           (and-let* ((dir (tabulated-list-get-id)))
-             (forge-get-repository :dir dir)))
+      (and-let ((_(derived-mode-p 'forge-repository-list-mode))
+                (id (tabulated-list-get-id)))
+        (forge-get-repository :id id))
+      (and-let ((_(derived-mode-p 'magit-repolist-mode))
+                (dir (tabulated-list-get-id)))
+        (forge-get-repository :dir dir))
       (and demand (user-error "No repository at point"))))
 
 (defun forge--repo-for-thingatpt ()
   (or (magit-section-value-if 'forge-repo)
-      (and-let* ((topic (magit-section-value-if '(issue pullreq))))
+      (and-let ((topic (magit-section-value-if '(issue pullreq))))
         (forge-get-repository topic))
       (and (not forge-buffer-unassociated-p)
            (or (forge-buffer-repository)
                (forge-get-repository :known? nil 'notatpt)))))
 
 (defun forge-buffer-repository ()
-  (and-let* ((id forge-buffer-repository))
+  (and-let ((id forge-buffer-repository))
     (forge-get-repository :id id)))
 
 (defun forge-set-buffer-repository ()
   "Initialize the value of variable `forge-buffer-repository'."
   (unless forge-buffer-repository
-    (and-let* ((repo (forge-get-repository :known?)))
-      (setq forge-buffer-repository (oref repo id)))))
+    (and$ (forge-get-repository :known?)
+          (setq forge-buffer-repository (oref $ id)))))
 
 (add-hook 'magit-mode-hook #'forge-set-buffer-repository)
 
@@ -340,16 +345,16 @@ If `default-directory' is within one of REPO's worktrees, record that
 location in its `worktree' slot and return it.  Otherwise, if a worktree
 has been recorded before, validate that.  If it still is a worktree of
 REPO, return it, else set the slot to nil and return nil."
-  (if-let (((forge-repository-equal
-             repo (forge-get-repository :dir default-directory)))
-           (current-tree (magit-toplevel)))
-      (oset repo worktree current-tree)
-    (and-let* ((saved-tree (oref repo worktree)))
-      (and (file-accessible-directory-p saved-tree)
-           (if (forge-repository-equal
-                repo (forge-get-repository :dir saved-tree))
-               saved-tree
-             (oset repo worktree nil))))))
+  (cond-let*
+    ([_(forge-repository-equal
+        repo (forge-get-repository :dir default-directory))]
+     [current-tree (magit-toplevel)]
+     (oset repo worktree current-tree))
+    ([saved-tree (oref repo worktree)]
+     [_(file-accessible-directory-p saved-tree)]
+     (if (forge-repository-equal repo (forge-get-repository :dir saved-tree))
+         saved-tree
+       (oset repo worktree nil)))))
 
 ;;;; List
 
@@ -433,11 +438,11 @@ forges and hosts."
                          (forge-sql [:select [githost owner name]
                                      :from repository]))
                  nil t nil nil
-                 (and-let* ((default (forge-get-repository :stub?)))
-                   (format "%s/%s @%s"
-                           (oref default owner)
-                           (oref default name)
-                           (oref default githost))))))
+                 (and$ (forge-get-repository :stub?)
+                       (format "%s/%s @%s"
+                               (oref $ owner)
+                               (oref $ name)
+                               (oref $ githost))))))
     (save-match-data
       (if (string-match "\\`\\(.+\\)/\\([^/]+\\) @\\(.+\\)\\'" choice)
           (forge-get-repository (list (match-string 3 choice)
@@ -486,31 +491,6 @@ forges and hosts."
        (?p . ,path)
        (?P . ,(string-replace "/" "%2F" path))))))
 
-(defun forge--set-field-callback (topic &optional preserve-status)
-  (let ((status (oref topic status)))
-    (lambda (&rest _)
-      (forge--pull-topic
-       (forge-get-repository topic)
-       topic
-       :callback (lambda ()
-                   ;; Necessary when setting a discussion field because
-                   ;; the API provides even less information about the
-                   ;; status of discussions compared to other topics and
-                   ;; as a result we would otherwise always switch the
-                   ;; status to `unread'.  This is not needed for every
-                   ;; modification of a discussion because some of them
-                   ;; (e.g., setting labels) do not cause `updated_at' to
-                   ;; be bumped; this second defect cancels out the first
-                   ;; when it comes to this function.
-                   (when preserve-status
-                     (oset topic status status))
-                   (forge-refresh-buffer)
-                   (when (transient-active-prefix
-                          '(forge-topic-menu
-                            forge-topics-menu
-                            forge-notifications-menu))
-                     (transient--refresh-transient)))))))
-
 (defvar forge--mode-line-buffer nil)
 
 (defun forge--msg (repo echo done format &rest args)
@@ -544,14 +524,9 @@ forges and hosts."
 
 (defun forge--ghub-type-symbol (class)
   (pcase-exhaustive class
-    ;; This package does not define a `forge-gitlab-http-repository'
-    ;; class, but we used to suggest at #9 that users define such a class
-    ;; if they must connect to a Gitlab instance that uses http instead
-    ;; of https.  Doing that isn't necessary anymore, but we have to keep
-    ;; supporting it here.  It is now sufficient to add an entry to
-    ;; `ghub-insecure-hosts'.
-    ((or 'forge-gitlab-repository 'forge-gitlab-http-repository) 'gitlab)
     ('forge-github-repository    'github)
+    ('forge-gitlab-repository    'gitlab)
+    ('forge-forgejo-repository   'forgejo)
     ('forge-gitea-repository     'gitea)
     ('forge-gogs-repository      'gogs)
     ('forge-bitbucket-repository 'bitbucket)))
@@ -559,8 +534,11 @@ forges and hosts."
 ;;; _
 ;; Local Variables:
 ;; read-symbol-shorthands: (
-;;   ("partial" . "llama--left-apply-partially")
-;;   ("rpartial" . "llama--right-apply-partially"))
+;;   ("and$"          . "cond-let--and$")
+;;   ("and-let"       . "cond-let--and-let")
+;;   ("if-let"        . "cond-let--if-let")
+;;   ("when-let"      . "cond-let--when-let")
+;;   ("partial"       . "llama--left-apply-partially"))
 ;; End:
 (provide 'forge-repo)
 ;;; forge-repo.el ends here
