@@ -22,6 +22,7 @@
 
 ;;; Code:
 
+(require 'bookmark)
 (require 'bug-reference)
 (require 'eieio-custom)
 (require 'markdown-mode)
@@ -336,24 +337,27 @@ A face attribute should be used that is not already used by any
   topic)
 
 (cl-defmethod forge-get-topic ((repo forge-repository) number-or-id)
-  (if (and (numberp number-or-id)
-           (< number-or-id 0))
-      (forge-get-pullreq repo (abs number-or-id))
-    (or (forge-get-discussion number-or-id)
-        (forge-get-issue number-or-id)
-        (forge-get-pullreq number-or-id))))
+  (cond ((stringp number-or-id)
+         (or (forge-get-discussion number-or-id)
+             (forge-get-issue      number-or-id)
+             (forge-get-pullreq    number-or-id)))
+        ((< number-or-id 0)
+         (forge-get-pullreq repo (abs number-or-id)))
+        ((forge-get-discussion repo number-or-id))
+        ((forge-get-issue      repo number-or-id))
+        ((forge-get-pullreq    repo number-or-id))))
 
 (cl-defmethod forge-get-topic ((number integer))
   (if (< number 0)
       (forge-get-pullreq (abs number))
     (or (forge-get-discussion number)
-        (forge-get-issue number)
-        (forge-get-pullreq number))))
+        (forge-get-issue      number)
+        (forge-get-pullreq    number))))
 
 (cl-defmethod forge-get-topic ((id string))
   (or (forge-get-discussion id)
-      (forge-get-issue id)
-      (forge-get-pullreq id)))
+      (forge-get-issue      id)
+      (forge-get-pullreq    id)))
 
 ;;;; Current
 
@@ -376,10 +380,10 @@ an error."
 
 (put 'forge-topic 'thing-at-point #'forge-thingatpt--topic)
 (defun forge-thingatpt--topic ()
-  (and-let* (((thing-at-point-looking-at "\\([#!]\\)\\([0-9]+\\)\\_>"))
-             (prefix (match-string-no-properties 1))
-             (number (string-to-number (match-string-no-properties 2)))
-             (repo (forge--repo-for-thingatpt)))
+  (and-let ((_(thing-at-point-looking-at "\\([#!]\\)\\([0-9]+\\)\\_>"))
+            (prefix (match-string-no-properties 1))
+            (number (string-to-number (match-string-no-properties 2)))
+            (repo (forge--repo-for-thingatpt)))
     (cond ((equal prefix "#")
            (forge-get-topic repo number))
           ((forge-gitlab-repository--eieio-childp repo)
@@ -457,8 +461,15 @@ State is the \"public condition\".  I.e., is the topic still open?"
                                        nil))))
                 :custom (choice
                          (const open)
+                         (const closed)
                          (const (completed merged))
+                         (const completed)
+                         (const merged)
                          (const (unplanned duplicate outdated rejected))
+                         (const unplanned)
+                         (const duplicate)
+                         (const outdated)
+                         (const rejected)
                          (const :tag "all (nil)" nil)))
    (status      :documentation "\
 Limit list based on topic (private) status.
@@ -523,23 +534,29 @@ current Forge database."
 Limit list to topics created by given user."
                 :initarg :author
                 :initform nil
-                :label "Author (username)"
+                :label "Author"
                 :type (or string null)
-                :custom string)
+                :custom (choice
+                         (string :tag "username")
+                         (const :tag "no filter (nil)" nil)))
    (assignee    :documentation "\
 Limit list to topics assigned to given user."
                 :initarg :assignee
                 :initform nil
-                :label "Assignee (username)"
+                :label "Assignee"
                 :type (or string null)
-                :custom string)
+                :custom (choice
+                         (string :tag "username")
+                         (const :tag "no filter (nil)" nil)))
    (reviewer    :documentation "\
 Limit list to topics for which a review by the given user was requested."
                 :initarg :reviewer
                 :initform nil
-                :label "Reviewer (username)"
+                :label "Reviewer"
                 :type (or string null)
-                :custom string)
+                :custom (choice
+                         (string :tag "username")
+                         (const :tag "no filter (nil)" nil)))
    (global      :documentation "Whether to list topics for all repositories."
                 :initarg :global
                 :initform nil
@@ -645,9 +662,6 @@ Limit list to topics for which a review by the given user was requested."
            [:join    pullreq-mark :on (= pullreq-mark:pullreq        topic:id)
             :join            mark :on (= mark:id              pullreq-mark:id)]))
       ,@(pcase (and assignee type)
-          ('discussion
-           [:join discussion-assignee :on (= discussion-assignee:discussion  topic:id)
-            :join            assignee :on (= assignee:id       discussion-assignee:id)])
           ('issue
            [:join      issue-assignee :on (= issue-assignee:issue            topic:id)
             :join            assignee :on (= assignee:id            issue-assignee:id)])
@@ -675,8 +689,10 @@ Limit list to topics for which a review by the given user was requested."
        ,@(and marks     `((or ,@(mapcar (##`(=  mark:name ,%))  marks))))
        ,@(and saved     '((= topic:saved-p  't)))
        ,@(and author    `((= topic:author   ,author)))
-       ,@(and assignee  `((= assignee:login ,assignee)))
-       ,@(and reviewer  `((= assignee:login ,reviewer))))
+       ,@(and assignee (memq type '(issue pullreq))
+              `((= assignee:login ,assignee)))
+       ,@(and reviewer (eq type 'pullreq)
+              `((= assignee:login ,reviewer))))
       :order-by [,(pcase order
                     ('newest            '(desc topic:number))
                     ('oldest            '(asc  topic:number))
@@ -840,7 +856,7 @@ can be selected from the start."
   (let ((crm-separator ","))
     (magit-completing-read-multiple
      "Labels: "
-     (forge--format-labels (and obj (forge-get-repository obj)))
+     (forge--format-labels (forge-get-repository (or obj :tracked)))
      nil t
      (and (cl-typep obj 'forge-topic)
           (forge--format-labels obj crm-separator)))))
@@ -906,8 +922,8 @@ can be selected from the start."
        ,(pcase state
           ('open                    'forge-topic-slug-open)
           ((or 'completed 'merged)  'forge-topic-slug-completed)
-          ((or 'unplanned 'duplicate 'rejected)
-           'forge-topic-slug-unplanned))))))
+          ((or 'unplanned 'outdated 'duplicate 'rejected)
+           'forge-topic-slug-expunged))))))
 
 (defun forge--format-topic-refs (topic)
   (pcase-let
@@ -951,7 +967,7 @@ can be selected from the start."
             ,(pcase (list (eieio-object-class topic) state)
                (`(forge-discussion  open)       'forge-discussion-open)
                (`(forge-discussion  completed)  'forge-discussion-completed)
-               (`(forge-discussion  unplanned)  'forge-discussion-expunged)
+               (`(forge-discussion  outdated)   'forge-discussion-expunged)
                (`(forge-discussion  duplicate)  'forge-discussion-expunged)
                (`(forge-issue       open)       'forge-issue-open)
                (`(forge-issue       completed)  'forge-issue-completed)
@@ -966,18 +982,18 @@ can be selected from the start."
 
 (defun forge--format-topic-category (topic)
   (and-let* ((id (oref topic category))
-             (str (caar (forge-sql [:select [name]
-                                    :from discussion-category
-                                    :where (= id $s1)]
-                                   id))))
+             (str (forge-sql1 [:select [name]
+                               :from discussion-category
+                               :where (= id $s1)]
+                              id)))
     (magit--propertize-face str 'forge-topic-label)))
 
 (defun forge--format-topic-milestone (topic)
   (and-let* ((id (oref topic milestone))
-             (str (caar (forge-sql [:select [title]
-                                    :from milestone
-                                    :where (= id $s1)]
-                                   id))))
+             (str (forge-sql1 [:select [title]
+                               :from milestone
+                               :where (= id $s1)]
+                              id)))
     (magit--propertize-face str 'forge-topic-label)))
 
 (defun forge--format-labels (&optional arg concat)
@@ -1016,17 +1032,17 @@ can be selected from the start."
       (mapcar format labels))))
 
 (defun forge--format-marks (&optional arg concat)
-  (and-let* ((marks (if (forge-topic--eieio-childp arg)
-                        (oref arg marks)
-                      ;; Unlike labels, marks are not repo-specific.
-                      (when (forge-repository-p arg) (setq arg nil))
-                      (forge-sql-cdr `[:select * :from mark
-                                       ,@(and arg '(:where (in name $v1)))
-                                       :order-by [(asc name)]]
-                                     (vconcat arg))))
-             (format (pcase-lambda (`(,_id ,name ,face ,_description))
-                       (magit--propertize-face
-                        name (list face 'forge-topic-label)))))
+  (and-let ((marks (if (forge-topic--eieio-childp arg)
+                       (oref arg marks)
+                     ;; Unlike labels, marks are not repo-specific.
+                     (when (forge-repository-p arg) (setq arg nil))
+                     (forge-sql-cdr `[:select * :from mark
+                                      ,@(and arg '(:where (in name $v1)))
+                                      :order-by [(asc name)]]
+                                    (vconcat arg))))
+            (format (pcase-lambda (`(,_id ,name ,face ,_description))
+                      (magit--propertize-face
+                       name (list face 'forge-topic-label)))))
     (if concat
         (mapconcat format marks (if (stringp concat) concat " "))
       (mapcar format marks))))
@@ -1038,7 +1054,7 @@ can be selected from the start."
      (pcase (list (if (forge-issue-p topic) 'issue 'pullreq) state)
        ('(discussion  open)       'forge-discussion-open)
        ('(discussion  completed)  'forge-discussion-completed)
-       ('(discussion  unplanned)  'forge-discussion-expunged)
+       ('(discussion  outdated)   'forge-discussion-expunged)
        ('(discussion  duplicate)  'forge-discussion-expunged)
        ('(issue       open)       'forge-issue-open)
        ('(issue       completed)  'forge-issue-completed)
@@ -1057,19 +1073,31 @@ can be selected from the start."
        ('pending 'forge-topic-pending)
        ('done    'forge-topic-done)))))
 
-(defun forge--format-topic-assignees (topic)
-  (and-let* ((assignees (oref topic assignees)))
+(defun forge--format-topic-assignees (arg)
+  (and-let ((assignees
+             (cond ((eieio-object-p arg)
+                    (oref arg assignees))
+                   ((forge-buffer-repository)
+                    (forge-sql-cdr [:select * :from assignee
+                                    :where
+                                    (and (= repository $s1)
+                                         (in login $v2))
+                                    :order-by [(asc login)]]
+                                   forge-buffer-repository
+                                   (vconcat arg))))))
     (mapconcat #'forge--format-person assignees ", ")))
 
 (defun forge--format-topic-review-requests (topic)
-  (and-let* ((review-requests (oref topic review-requests)))
-    (mapconcat #'forge--format-person review-requests ", ")))
+  (and$ (oref topic review-requests)
+        (mapconcat #'forge--format-person $ ", ")))
 
 (defun forge--format-person (person)
-  (pcase-let ((`(,_id ,login ,name) person))
-    (format "%s%s (@%s)"
-            (forge--format-avatar login)
-            name login)))
+  (pcase-let* ((`(,_id ,login ,name) person)
+               (avatar (forge--format-avatar login)))
+    (propertize (if name
+                    (format "%s%s (@%s)" avatar name login)
+                  (format "%s@%s" avatar login))
+                'face 'transient-value)))
 
 (defun forge--format-avatar (person)
   (if forge-format-avatar-function
@@ -1094,11 +1122,11 @@ can be selected from the start."
   "Insert a list of topics, according to PREPARE.
 
 This function is not intended to be added to section hooks directly.
-Instead create a function, which calls this function, and add wrapper
-to the section hook.
+Instead create a function, which calls this function, and add that
+wrapper to the mode's section hook.
 
 PREPARE is a function which takes one arguments the repository object,
-and must return an filter object of type `forge--topics-spec' or nil.
+and must return a filter object of type `forge--topics-spec' or nil.
 Insert no topics if PREPARE returns nil, or if the current repository
 isn't tracked or Forge hasn't been fully setup yet (in the latter two
 cases don't even call PREPARE).
@@ -1111,11 +1139,31 @@ See `forge--topics-spec' for the valid slots and their values.
 HEADING is used as the heading of the list section and TYPE is used as
 its type.  TYPE should be a symbol of the form `SUBSET-KIND', where KIND
 is one of `topics', `issues' or `pullreqs', and SUBSET should describe
-what subset of KIND is being listed."
+what subset of KIND is being listed.
+
+For example, to insert a list of issues assigned to you use something
+like:
+
+  (defun my-forge-insert-assigned-issues ()
+    \"Insert a list of issues that are assigned to me.\"
+    (forge-insert-topics \\='assigned-issues \"Assigned issues\"
+      (lambda (repo)
+        (and-let* ((me (ghub--username repo)))
+          (forge--topics-spec :type \\='issue :active t
+                              :assignee me)))))
+
+  (magit-add-section-hook \\='magit-status-sections-hook
+                          #\\='my-forge-insert-assigned-issues
+                          #\\='forge-insert-issues)
+
+Grep Forge for more examples.
+
+Alternatively you can use `forge-topics-setup-buffer' to list a set
+of topics in a dedicated buffer."
   (declare (indent defun))
-  (when-let (((forge-db t))
-             (repo (forge-get-repository :tracked?))
-             (spec (funcall prepare repo)))
+  (when-let* ((_(forge-db t))
+              (repo (forge-get-repository :tracked?))
+              (spec (funcall prepare repo)))
     (forge--insert-topics type heading (forge--list-topics spec repo))))
 
 (defun forge--insert-topics (type heading topics)
@@ -1141,8 +1189,7 @@ what subset of KIND is being listed."
     (insert "\n")
     (magit-log-format-author-margin
      (oref topic author)
-     (format-time-string "%s" (parse-iso8601-time-string (oref topic created)))
-     t)
+     (format-time-string "%s" (parse-iso8601-time-string (oref topic created))))
     (when (and (slot-exists-p topic 'merged)
                (not (oref topic merged)))
       (magit-insert-heading)
@@ -1150,7 +1197,7 @@ what subset of KIND is being listed."
         (forge--insert-pullreq-commits topic)))))
 
 (defun forge--insert-topic-labels (topic &optional separate)
-  (and-let* ((labels (oref topic labels)))
+  (and-let ((labels (oref topic labels)))
     (prog1 t
       (pcase-dolist (`(,_id ,name ,color ,description) labels)
         (let* ((background (forge--sanitize-color color))
@@ -1168,7 +1215,7 @@ what subset of KIND is being listed."
               (overlay-put o 'help-echo description))))))))
 
 (defun forge--insert-topic-marks (topic &optional separate)
-  (and-let* ((marks (oref topic marks)))
+  (and-let ((marks (oref topic marks)))
     (prog1 t
       (pcase-dolist (`(,_id ,name ,face ,description) marks)
         (if separate (insert " ") (setq separate t))
@@ -1274,15 +1321,17 @@ This mode itself is never used directly."
 (defun forge-topic-setup-buffer (topic)
   (let* ((repo (forge-get-repository topic))
          (name (format "*forge: %s %s*" (oref repo slug) (oref topic slug)))
-         (magit-generate-buffer-name-function (lambda (_mode _value) name)))
-    (magit-setup-buffer-internal
-     (pcase-exhaustive (eieio-object-class topic)
-       ('forge-discussion #'forge-discussion-mode)
-       ('forge-issue      #'forge-issue-mode)
-       ('forge-pullreq    #'forge-pullreq-mode))
-     t `((forge-buffer-topic ,topic))
-     name (or (forge-get-worktree repo) "/"))
-    (forge-topic-mark-read topic)))
+         (magit-generate-buffer-name-function (lambda (_mode _value) name))
+         (mode (pcase-exhaustive (eieio-object-class topic)
+                 ('forge-discussion #'forge-discussion-mode)
+                 ('forge-issue      #'forge-issue-mode)
+                 ('forge-pullreq    #'forge-pullreq-mode)))
+         (buffer (magit-setup-buffer mode t
+                   :buffer name
+                   :directory (or (forge-get-worktree repo) "/")
+                   (forge-buffer-topic topic))))
+    (forge-topic-mark-read topic)
+    buffer))
 
 (defun forge-topic-refresh-buffer ()
   (let ((topic (closql-reload forge-buffer-topic)))
@@ -1290,8 +1339,10 @@ This mode itself is never used directly."
     (magit-set-header-line-format (forge--format-topic-line topic))
     (magit-insert-section (topicbuf)
       (magit-insert-headers
-       (intern (format "%s-headers-hook"
-                       (substring (symbol-name major-mode) 0 -5))))
+       (pcase major-mode
+         ('forge-discussion-mode 'forge-discussion-headers-hook)
+         ('forge-issue-mode      'forge-issue-headers-hook)
+         ('forge-pullreq-mode    'forge-pullreq-headers-hook)))
       (when (forge-pullreq-p topic)
         (magit-insert-section (pullreq topic)
           (magit-insert-heading "Commits")
@@ -1340,15 +1391,15 @@ This mode itself is never used directly."
       (setq heading (concat "    " heading)))
     (font-lock-append-text-property
      0 (length heading)
-     'font-lock-face (cond
-                      ((and (forge-discussion-p topic)
-                            (and-let* ((answer (oref topic answer)))
-                              (equal (oref post their-id)
-                                     (forge--their-id answer))))
-                       'forge-discussion-answer-heading)
-                      ((forge-discussion-reply-p post)
-                       '(magit-dimmed magit-diff-hunk-heading))
-                      ('magit-diff-hunk-heading))
+     'font-lock-face (cond-let*
+                       ([_(forge-discussion-p topic)]
+                        [answer (oref topic answer)]
+                        [_(equal (oref post their-id)
+                                 (forge--their-id answer))]
+                        'forge-discussion-answer-heading)
+                       ((forge-discussion-reply-p post)
+                        '(magit-dimmed magit-diff-hunk-heading))
+                       ('magit-diff-hunk-heading))
      heading)
     (magit-insert-heading heading)))
 
@@ -1361,6 +1412,28 @@ This mode itself is never used directly."
 (cl-defmethod magit-buffer-value (&context (major-mode forge-topic-mode))
   (oref forge-buffer-topic slug))
 
+;;; Bookmarks
+
+(cl-defmethod magit-bookmark-name
+  (&context (major-mode forge-topic-mode))
+  (concat (oref (forge-get-repository forge-buffer-topic) slug)
+          (oref forge-buffer-topic slug)))
+
+(cl-defmethod magit-bookmark-get-value
+  (bookmark &context (major-mode forge-topic-mode))
+  (bookmark-prop-set bookmark 'forge-topic (oref forge-buffer-topic id)))
+
+(cl-defmethod magit-bookmark-get-buffer-create
+  (bookmark (_mode (derived-mode forge-topic-mode)))
+  (let ((magit-display-buffer-function #'identity)
+        (magit-display-buffer-noselect t))
+    (forge-topic-setup-buffer
+     (forge-get-topic (bookmark-prop-get bookmark 'forge-topic)))))
+
+(put 'forge-discussion-mode 'magit-bookmark-variables t)
+(put 'forge-issue-mode      'magit-bookmark-variables t)
+(put 'forge-pullreq-mode    'magit-bookmark-variables t)
+
 ;;; Headers
 
 (cl-defmacro forge--define-topic-header
@@ -1428,7 +1501,7 @@ This mode itself is never used directly."
 ;;; Commands
 ;;;; Groups
 
-(defconst forge--lists-group
+(transient-define-group forge--lists-group
   ["List"
    ("l r" "repositories"  forge-list-repositories)
    ("l n" "notifications" forge-list-notifications)
@@ -1436,7 +1509,7 @@ This mode itself is never used directly."
    ("l t" "topics"        forge-list-topics)
    ""])
 
-(defconst forge--topic-menus-group
+(transient-define-group forge--topic-menus-group
   ["Menu"
    ("m s" "edit"      forge-topic-menu)
    ("m f" "filter"    forge-topics-menu)
@@ -1446,7 +1519,7 @@ This mode itself is never used directly."
    ("m c" "configure" forge-configure)
    """"])
 
-(defconst forge--topic-set-state-group
+(transient-define-group forge--topic-set-state-group
   [:description (##if forge--show-topic-legend "Set public state" "Set state")
    ("o" forge-topic-state-set-open)
    ("c" forge-chatter-state-set-completed)
@@ -1456,31 +1529,31 @@ This mode itself is never used directly."
    ("M" forge-pullreq-state-set-merged)
    ("R" forge-pullreq-state-set-rejected)])
 
-(defconst forge--topic-set-status-group
+(transient-define-group forge--topic-set-status-group
   [:description (##if forge--show-topic-legend "Set private status" "Set status")
    ("u" forge-topic-status-set-unread)
    ("p" forge-topic-status-set-pending)
    ("d" forge-topic-status-set-done)])
 
-(defconst forge--topic-legend-group
-  '(["Legend" :if-non-nil forge--show-topic-legend
-     (:info* (##propertize "open discussion"      'face 'forge-discussion-open))
-     (:info* (##propertize "completed discussion" 'face 'forge-discussion-completed))
-     (:info* (##propertize "expunged discussion"  'face 'forge-discussion-expunged))]
-    ["" :if-non-nil forge--show-topic-legend
-     (:info* (##propertize "open issue"       'face 'forge-issue-open))
-     (:info* (##propertize "completed issue"  'face 'forge-issue-completed))
-     (:info* (##propertize "expunged issue"   'face 'forge-issue-expunged))]
-    ["" :if-non-nil forge--show-topic-legend
-     (:info* (##propertize "open pullreq"     'face 'forge-pullreq-open))
-     (:info* (##propertize "merged pullreq"   'face 'forge-pullreq-merged))
-     (:info* (##propertize "rejected pullreq" 'face 'forge-pullreq-rejected))]
-    ["" :if-non-nil forge--show-topic-legend
-     (:info* (##propertize "unread"           'face 'forge-topic-unread))
-     (:info* (##propertize "pending"          'face 'forge-topic-pending))
-     (:info* (##propertize "done"             'face 'forge-topic-done))]
-    ["" :if-non-nil forge--show-topic-legend
-     (:info* (##propertize "draft"            'face 'forge-pullreq-draft))]))
+(transient-define-group forge--topic-legend-group
+  ["Legend" :if-non-nil forge--show-topic-legend
+   (:info* (##propertize "open discussion"      'face 'forge-discussion-open))
+   (:info* (##propertize "completed discussion" 'face 'forge-discussion-completed))
+   (:info* (##propertize "expunged discussion"  'face 'forge-discussion-expunged))]
+  ["" :if-non-nil forge--show-topic-legend
+   (:info* (##propertize "open issue"       'face 'forge-issue-open))
+   (:info* (##propertize "completed issue"  'face 'forge-issue-completed))
+   (:info* (##propertize "expunged issue"   'face 'forge-issue-expunged))]
+  ["" :if-non-nil forge--show-topic-legend
+   (:info* (##propertize "open pullreq"     'face 'forge-pullreq-open))
+   (:info* (##propertize "merged pullreq"   'face 'forge-pullreq-merged))
+   (:info* (##propertize "rejected pullreq" 'face 'forge-pullreq-rejected))]
+  ["" :if-non-nil forge--show-topic-legend
+   (:info* (##propertize "unread"           'face 'forge-topic-unread))
+   (:info* (##propertize "pending"          'face 'forge-topic-pending))
+   (:info* (##propertize "done"             'face 'forge-topic-done))]
+  ["" :if-non-nil forge--show-topic-legend
+   (:info* (##propertize "draft"            'face 'forge-pullreq-draft))])
 
 (defvar forge--show-topic-legend t)
 
@@ -1585,9 +1658,9 @@ This mode itself is never used directly."
    (inapt-face
     :initform (lambda (obj)
                 (with-slots (getter state) (transient-suffix-object)
-                  (if (and (not (forge-region-topics))
-                           (and-let* ((topic (funcall getter)))
-                             (eq (oref topic state) state)))
+                  (if (and-let ((_(not (forge-region-topics)))
+                                (topic (funcall getter)))
+                        (eq (oref topic state) state))
                       'forge-suffix-active
                     'transient-inapt-suffix))))))
 
@@ -1634,13 +1707,32 @@ This mode itself is never used directly."
   :if #'forge-current-discussion)
 
 (transient-define-suffix forge-pullreq-state-set-merged ()
-  "If the current pull-request is merged, then visualize that."
+  "Merge the current pull-request into its target.
+Prompt the user to either use the API to perform the merge or use Git.
+I recommend you only use the API if your organization enforces that
+inferior process."
   :class 'forge--topic-set-state-command
   :state 'merged
   :getter #'forge-current-pullreq
   :if #'forge-current-pullreq
+  :transient nil
   (interactive)
-  (message "Please use a merge command for this"))
+  (let ((pullreq (forge-current-pullreq)))
+    (if (magit-read-char-case (format "Merge #%s " (oref pullreq number)) t
+          (?g "using [g]it (recommended)" t)
+          (?a "using [a]pi" nil))
+        (let ((branch (or (forge--pullreq-branch-active pullreq)
+                          (forge--branch-pullreq pullreq))))
+          (if-let ((upstream (magit-get-local-upstream-branch branch)))
+              (if (zerop (magit-call-git "checkout" upstream))
+                  (magit--merge-absorb
+                   branch (magit-merge-arguments)
+                   ;; Users might be surprised that we
+                   ;; aren't done yet, so drop a hint.
+                   "Inspect the result, and if satisfied push")
+                (user-error "Could not checkout %S" upstream))
+            (user-error "No upstream configured for %S" branch)))
+      (forge-merge pullreq (forge-select-merge-method)))))
 
 (transient-define-suffix forge-pullreq-state-set-rejected ()
   "Set the state of the current pull-request to `rejected'."
@@ -1791,7 +1883,13 @@ This mode itself is never used directly."
   "Mark the post at point as the answer to the current question.
 When point is on the answer, then unmark it and mark no other."
   :class 'forge--topic-set-slot-command :slot 'answer
-  :inapt-if-not #'forge-current-discussion
+  :inapt-if-not (lambda ()
+                  (and-let* ((discussion (forge-current-discussion))
+                             (category (oref discussion category)))
+                    (forge-sql1 [:select answerable-p
+                                 :from discussion-category
+                                 :where (= id $s1)]
+                                category)))
   :description (##forge--format-boolean 'answer "answered")
   :reader #'forge--select-discussion-answer)
 
@@ -1818,7 +1916,16 @@ When point is on the answer, then unmark it and mark no other."
       (fill-region (point-min) (point-max)))
     (when indent
       (indent-rigidly (point-min) (point-max) indent))
-    (buffer-string)))
+    (let* ((string (buffer-string))
+           (beg 0)
+           (end (length string)))
+      (while (< beg end)
+        (let ((pos (next-single-property-change beg 'face string end))
+              (val (get-text-property beg 'face string)))
+          (put-text-property beg pos 'font-lock-face val string)
+          (remove-list-of-text-properties beg pos '(face) string)
+          (setq beg pos)))
+      string)))
 
 (defun forge--markdown-translate-filename-function (file)
   (if (string-match-p "\\`https?://" file)
@@ -1831,120 +1938,108 @@ When point is on the answer, then unmark it and mark no other."
 
 ;;; Templates
 
-(defun forge--topic-parse-buffer (&optional file)
-  (save-match-data
-    (save-excursion
-      (goto-char (point-min))
-      (let ((alist (save-excursion (forge--topic-parse-yaml))))
-        (if alist
-            (setf (alist-get 'yaml alist) t)
-          (setq alist (save-excursion (forge--topic-parse-plain))))
-        (setf (alist-get 'file alist) file)
-        (setf (alist-get 'text alist) (magit--buffer-string nil nil ?\n))
-        (when (and file (not (alist-get 'prompt alist)))
-          (setf (alist-get 'prompt alist)
-                (file-name-sans-extension (file-name-nondirectory file))))
-        ;; If there is a yaml front-matter, then it is supposed
-        ;; to have a `title' field, but this may not be the case.
-        (when (and (not file)
-                   (not (alist-get 'title alist)))
-          (setf (alist-get 'title alist)
-                (read-string "Title: ")))
-        alist))))
+(defun forge--topic-template (repo class)
+  (unless repo
+    (setq repo (forge-get-repository :tracked)))
+  (let* ((templates (forge--topic-templates repo class))
+         (template
+          (if (cdr templates)
+              (let ((c (magit-completing-read
+                        (pcase class
+                          ('forge-issue   "Select issue template")
+                          ('forge-pullreq "Select pull-request template"))
+                        (mapcar (##alist-get 'prompt %) templates)
+                        nil t)))
+                (seq-find (##equal (alist-get 'prompt %) c) templates))
+            (car templates))))
+    (if-let ((url (alist-get 'url template)))
+        (if (string-match (forge--format repo "\
+\\`https://%h/[^/]+/[^/]+/discussions\\(?:/categories/\\(.+\\)\\)?")
+                          url)
+            `((type . forge-discussion)
+              (category . ,(or (match-string 1 url)
+                               (forge-read-topic-category
+                                nil "Category for new discussion"))))
+          `((type . redirect) ,@template))
+      `((type . ,class) ,@template))))
 
-(defun forge--topic-parse-yaml ()
-  (let (alist beg end)
-    (when (looking-at "^---[\s\t]*$")
-      (forward-line)
-      (setq beg (point))
-      (when (re-search-forward "^---[\s\t]*$" nil t)
-        (setq end (match-beginning 0))
-        (setq alist (yaml-parse-string
-                     (buffer-substring-no-properties beg end)
-                     :object-type 'alist
-                     :sequence-type 'list
-                     :false-object nil))
-        (let-alist alist
-          (when (and .name .about)
-            (setf (alist-get 'prompt alist)
-                  (format "[%s] %s" .name .about)))
-          (when (and .labels (atom .labels))
-            (setf (alist-get 'labels alist) (list .labels)))
-          (when (and .assignees (atom .assignees))
-            (setf (alist-get 'assignees alist) (list .assignees))))
-        (forward-line)
-        (when (and (not (alist-get 'title alist))
-                   (looking-at "^\n?#*"))
-          (goto-char (match-end 0))
-          (setf (alist-get 'title alist)
-                (string-trim
-                 (magit--buffer-string (point) (line-end-position) t)))
-          (forward-line))
-        (setf (alist-get 'body alist)
-              (string-trim (magit--buffer-string (point) nil ?\n)))))
-    alist))
+(defun forge--topic-templates (repo class)
+  (mapcan (lambda (file)
+            (with-temp-buffer
+              (magit-git-insert "cat-file" "-p" file)
+              (if (equal (file-name-nondirectory file) "config.yml")
+                  (forge--topic-parse-template-config)
+                (list (forge--topic-parse-template (file-name-base file))))))
+          (forge--topic-template-files repo class)))
 
-(defun forge--topic-parse-plain ()
-  (let (title body)
-    (when (looking-at "\\`#*")
-      (goto-char (match-end 0)))
-    (setq title (magit--buffer-string (point) (line-end-position) t))
-    (forward-line)
-    (setq body (magit--buffer-string (point) nil ?\n))
-    `((title . ,(string-trim title))
-      (body  . ,(string-trim body)))))
+(cl-defgeneric forge--topic-template-files (repo class))
 
-(defun forge--topic-parse-link-buffer ()
-  (save-match-data
-    (save-excursion
-      (goto-char (point-min))
-      (mapcar (lambda (alist)
-                (cons (cons 'prompt (concat (alist-get 'name alist) " -- "
-                                            (alist-get 'about alist)))
-                      alist))
-              (forge--topic-parse-yaml-links)))))
+(defun forge--topic-template-files-1 (repo suffix &rest paths)
+  (setq suffix (ensure-list suffix))
+  (let ((branch (forge--get-default-branch repo)))
+    (seq-keep (if suffix
+                  (##and (member (file-name-extension %) suffix)
+                         (concat branch ":" %))
+                (##concat branch ":" %))
+              (magit-git-items "ls-tree" "-z"
+                               "--full-tree" "--name-only"
+                               (and suffix "-r")
+                               branch "--" paths))))
 
-(defun forge--topic-parse-yaml-links ()
-  (alist-get 'contact_links
-             (yaml-parse-string (buffer-substring-no-properties
-                                 (point-min)
-                                 (point-max))
+(defun forge--topic-parse-template-config ()
+  (let-alist (yaml-parse-string (buffer-str)
                                 :object-type 'alist
-                                :sequence-type 'list)))
+                                :sequence-type 'list)
+    (nconc
+     (and (not (eq .blank_issues_enabled :false)) ;unset means true
+          `(((prompt . ,(concat (propertize "Blank issue" 'face 'bold)
+                                " — Create a new issue from scratch")))))
+     (mapcar (lambda (link)
+               `(,@link
+                 (prompt . ,(let-alist link
+                              (concat (propertize .name 'face 'bold)
+                                      " — " .about)))))
+             .contact_links))))
 
-(cl-defgeneric forge--topic-template-files (repo class)
-  "Return a list of topic template files for REPO and a topic of CLASS.")
-
-(cl-defgeneric forge--topic-template (repo class)
-  "Return a topic template alist for REPO and a topic of CLASS.
-If there are multiple templates, then the user is asked to select
-one of them.  It there are no templates, then return a very basic
-alist, containing just `text' and `position'.")
-
-(defun forge--topic-templates-data (repo class)
-  (let ((branch (oref repo default-branch)))
-    (mapcan (lambda (f)
-              (with-temp-buffer
-                (magit-git-insert "cat-file" "-p" (concat branch ":" f))
-                (if (equal (file-name-nondirectory f) "config.yml")
-                    (forge--topic-parse-link-buffer)
-                  (list (forge--topic-parse-buffer f)))))
-            (forge--topic-template-files repo class))))
-
-(cl-defmethod forge--topic-template ((repo forge-repository)
-                                     (class (subclass forge-topic)))
-  (let ((choices (and (not (eq class 'forge-discussion))
-                      (forge--topic-templates-data repo class))))
-    (if (cdr choices)
-        (let ((c (magit-completing-read
-                  (pcase class
-                    ('forge-discussion "Select discussion type")
-                    ('forge-issue      "Select issue template")
-                    ('forge-pullreq    "Select pull-request template"))
-                  (mapcar (##alist-get 'prompt %) choices)
-                  nil t)))
-          (seq-find (##equal (alist-get 'prompt %) c) choices))
-      (car choices))))
+(defun forge--topic-parse-template (name)
+  (goto-char (point-min))
+  (skip-chars-forward "\s\t\n\r")
+  (if-let ((beg (and (looking-at "^---[\s\t]*$")
+                     (point)))
+           (end (and (zerop (forward-line))
+                     (re-search-forward "^---[\s\t]*$" nil t)
+                     (match-beginning 0)))
+           (repoid (oref (forge-get-repository :tracked) id)))
+      (let-alist (yaml-parse-string (buffer-str beg end)
+                                    :object-type 'alist
+                                    :sequence-type 'list
+                                    :null-object nil
+                                    :false-object nil)
+        (when (stringp .name)
+          (setq name .name))
+        (setq name (propertize .name 'face 'bold))
+        `((prompt    . ,(if .about (format "%s — %s" name .about) name))
+          (title     . ,(and .title
+                             (stringp .title)
+                             (string-trim .title)))
+          (text      . ,(string-trim (buffer-str (point))))
+          ;; Prevent ad hock creation or previously unknown labels.
+          (labels    . ,(cl-intersection
+                         (ensure-list .labels)
+                         (forge-sql-car [:select name :from label
+                                         :where (= repository $s1)]
+                                        repoid)
+                         :test #'equal))
+          ;; Server errors on invalid assignees.
+          (assignees . ,(cl-intersection
+                         (ensure-list .assignees)
+                         (forge-sql-car [:select login :from assignee
+                                         :where (= repository $s1)]
+                                        repoid)
+                         :test #'equal))
+          (draft     . ,(and (booleanp .draft) .draft))))
+    `((prompt . ,(propertize name 'face 'bold))
+      (text   . ,(string-trim (buffer-str))))))
 
 ;;; Bug-Reference
 
@@ -1993,8 +2088,13 @@ modify `bug-reference-bug-regexp' if appropriate."
 ;;; _
 ;; Local Variables:
 ;; read-symbol-shorthands: (
-;;   ("partial" . "llama--left-apply-partially")
-;;   ("rpartial" . "llama--right-apply-partially"))
+;;   ("and$"          . "cond-let--and$")
+;;   ("and-let"       . "cond-let--and-let")
+;;   ("if-let"        . "cond-let--if-let")
+;;   ("when-let"      . "cond-let--when-let")
+;;   ("buffer-string" . "buffer-string")
+;;   ("buffer-str"    . "forge--buffer-substring-no-properties")
+;;   ("partial"       . "llama--left-apply-partially"))
 ;; End:
 (provide 'forge-topic)
 ;;; forge-topic.el ends here
